package com.selya.store.global.action;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts2.interceptor.ServletRequestAware;
import org.apache.struts2.interceptor.ServletResponseAware;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.Preparable;
import com.selya.store.global.utils.PagerBean;
import com.selya.store.global.utils.PagerMethod;
import com.selya.store.persistance.Carreto;
import com.selya.store.persistance.CarretoDAO;
import com.selya.store.persistance.Usuaris;

/**
 * 
 * @author Dani Martín
 *
 */
@SuppressWarnings("serial")
public class BaseAction extends ActionSupport implements Preparable,
		ServletRequestAware, ServletResponseAware {

	protected static final String SESSION_USER = "user";
	protected static final String SESSION_CARRETO = "carreto";
	protected static final String SESSION_LANGUAGE = "language";
	protected static final String DEFAULT_LANGUAGE = "ca_ES";
	
	protected static final Integer NUM_MAXINTERVALOS_DEFAULT = 5;
	protected static final Integer NUM_MAXELEMENTSPAGE_DEFAULT = 5;
	
	protected Logger logger = Logger.getLogger(this.getClass().getName());
	
	protected HttpServletRequest request;
	protected HttpServletResponse response;
	
    private Usuaris user;
    private String menu;
    private String language;
    
	protected PagerBean pagerBean;
	protected Integer page = 1;
	protected Integer numElements;
	
    @Override
	public void prepare() throws Exception {
		user = this.initUser(false);
		this.initCarreto();
		setLanguage(this.initLanguage());
		initCarreto();
	}
	
    protected void initCarreto(){
    	CarretoDAO carretoDAO = new CarretoDAO();
    	if(request.getSession().getAttribute(SESSION_CARRETO) == null){
    		if(user != null){
    			Carreto carreto = carretoDAO.getLastCarreto(user);
    			if(carreto != null)
    				request.getSession().setAttribute(SESSION_CARRETO, carreto);
    			else{
    				carreto = new Carreto();
    				carreto.setDataCreacio(new Date());
    				carreto.setDataModificacio(new Date());
    				carretoDAO.save(carreto);
    				carreto.setUsuaris(user);
    				carretoDAO.save(carreto);
    				request.getSession().setAttribute(SESSION_CARRETO, new Carreto());
    			}
    		}else{
    			Carreto carreto = new Carreto();
				carreto.setDataCreacio(new Date());
				carreto.setDataModificacio(new Date());
    			request.getSession().setAttribute(SESSION_CARRETO, new Carreto());
    		}
    	}
    }

	protected Usuaris initUser(boolean value) {
		Usuaris user = (value) ? null : (Usuaris)request.getSession().getAttribute(SESSION_USER);
		return user;
	}
	
	protected String initLanguage() {
		String language = (String) request.getSession().getAttribute(SESSION_LANGUAGE);
		if(language == null){
			language = DEFAULT_LANGUAGE;
		}
		return language;
	}
	
	@SuppressWarnings("rawtypes")
	protected List getResultsByPage(List contentIds, Integer numMaxIntervalos, Integer numMaxElementsPage, String attrName, String attr) {
		numElements = contentIds.size();
		PagerMethod pagerMethod = new PagerMethod(request,numElements,page,numMaxIntervalos,numMaxElementsPage, attrName, attr);
		pagerBean = pagerMethod.doPagerBean();
		
		int begin = (page-1)*pagerMethod.getNumMaxElementsPage();
		int end = page*pagerMethod.getNumMaxElementsPage();
		
		if(end > contentIds.size()){
			end = contentIds.size();
		}
		return contentIds.subList(begin,end);
	}
	
	@SuppressWarnings("rawtypes")
	protected List getResultsByPage(List contentIds, Integer numMaxIntervalos, Integer numMaxElementsPage) {
		numElements = contentIds.size();
		PagerMethod pagerMethod = new PagerMethod(request,numElements,page,numMaxIntervalos,numMaxElementsPage);
		pagerBean = pagerMethod.doPagerBean();
		
		int begin = (page-1)*pagerMethod.getNumMaxElementsPage();
		int end = page*pagerMethod.getNumMaxElementsPage();
		
		if(end > contentIds.size()){
			end = contentIds.size();
		}
		return contentIds.subList(begin,end);
	}

	@SuppressWarnings("rawtypes")
	protected List getResultsByPage(List contentIds, String attrName, String attr) {
		return getResultsByPage(contentIds,NUM_MAXINTERVALOS_DEFAULT,NUM_MAXELEMENTSPAGE_DEFAULT, attrName, attr);
	}
	
	@SuppressWarnings("rawtypes")
	protected List getResultsByPage(List contentIds) {
		return getResultsByPage(contentIds,NUM_MAXINTERVALOS_DEFAULT,NUM_MAXELEMENTSPAGE_DEFAULT);
	}
	
	/* SETTERS & GETTERS */
	
	@Override
	public void setServletResponse(HttpServletResponse response) {
		this.response = response;
	}

	@Override
	public void setServletRequest(HttpServletRequest request) {
		this.request = request;
	}

	public Usuaris getUser() {
		return user;
	}

	public void setUser(Usuaris user) {
		this.user = user;
		request.getSession().setAttribute(SESSION_USER, user);
	}

	public String getMenu() {
		return menu;
	}

	public void setMenu(String menu) {
		this.menu = menu;
	}

	public HttpServletRequest getRequest() {
		return request;
	}

	public HttpServletResponse getResponse() {
		return response;
	}

	/**
	 * @return the language
	 */
	public String getLanguage() {
		return language;
	}

	/**
	 * @param language the language to set
	 */
	public void setLanguage(String language) {
		this.language = language;
		request.getSession().setAttribute(SESSION_LANGUAGE, language);
	}

	public PagerBean getPagerBean() {
		return pagerBean;
	}

	public void setPagerBean(PagerBean pagerBean) {
		this.pagerBean = pagerBean;
	}

	public Integer getPage() {
		return page;
	}

	public void setPage(Integer page) {
		this.page = page;
	}

	public Integer getNumElements() {
		return numElements;
	}

	public void setNumElements(Integer numElements) {
		this.numElements = numElements;
	}

}
