package com.selya.store.global.utils;

import javax.servlet.http.HttpServletRequest;

public class PagerMethod {

	private long numElements;
	private int numIntervalos;
	private int numMaxIntervalos;
	private int numMaxElementsPage;
	private String action;
	private int pageActual;
	private int numElementsIntervalo;
	private PagerBean pagerBean;
	private int paginaMaxima;
	private int primeraPagina;
	private String attr;
	private String attrName;
		
	/**
	 * Constructora para poder crear la paginación
	 * @param request El Request de la petición de la página
	 * @param numElements Numero total de elementos.
	 * @param page Pagina actual (Por defecto 1).
	 * @param numMaxIntervalos numero máximo de páginas a mostrar
	 * @param numMaxElementsPage numero máximo de elementos por página
	 */
	public PagerMethod(HttpServletRequest request, long numElements, int page, int numMaxIntervalos, int numMaxElementsPage){
		
		this.numMaxIntervalos = numMaxIntervalos;
		this.numMaxElementsPage = numMaxElementsPage;
		// Prealizamos operaciones para instanciar el objeto
		this.preparePagerMethod(request, numElements, page);
	}
		
	/**
	 * Constructora para poder crear la paginación
	 * @param request El Request de la petición de la página
	 * @param numElements Numero total de elementos.
	 * @param page Pagina actual (Por defecto 1).
	 */
	public PagerMethod(HttpServletRequest request, long numElements, int page){
		
		// Recuperamos del config.properties el máximo de elementos
		// y el máximo de intervalos
		this.numMaxIntervalos = new Integer(30);
		this.numMaxElementsPage = new Integer(10);
		// Realizamos operaciones para instanciar el objeto
		this.preparePagerMethod(request, numElements, page);
		
	}
	
	public PagerMethod(HttpServletRequest request, long numElements, int page, int numMaxIntervalos, int numMaxElementsPage, String attrName, String attr){
		this.numMaxIntervalos = numMaxIntervalos;
		this.numMaxElementsPage = numMaxElementsPage;
		// Realizamos operaciones para instanciar el objeto
		this.preparePagerMethod(request, numElements, page, attrName, attr);
	}
	
	private void preparePagerMethod(HttpServletRequest request, long numElements, int page) {
		
		//Creamos la acción correspondiente
		this.action = DoUrl(request);
		
		// Creamos el bean a la que accederá la jsp y que 
		// contendra los datos necesarios.
		this.pagerBean = new PagerBean();
		this.numElements = numElements;
		this.pageActual = page;
		
		// Calculamos el numero de intervalos que necesitamos.
		this.numIntervalos = CalcularIntervalosTotal();
		this.paginaMaxima = this.numIntervalos;
		
		// Si el numero de intervalos es mayor que el máximo permitido
		if(numIntervalos>numMaxIntervalos){
			this.primeraPagina = this.pageActual-(this.numMaxIntervalos/2);
			
			// La primera página obligatoriamente es 1 o más
			if(this.primeraPagina<=1){
				this.primeraPagina=1;
			}
			
			// Si el número máximo de intervalos es impar
			else if(((this.numMaxIntervalos)%2==1)&&(this.numIntervalos-this.pageActual-(this.numMaxIntervalos/2)<0)){			
				// Al ser impar no podemos eleminar el 2 que multiplica con el 2 que divide
				this.primeraPagina = this.numIntervalos-2*(this.numMaxIntervalos/2);	
			}		
			// Si el número máximo de intervalos es par
			else if(((this.numMaxIntervalos)%2==0)&&(this.numIntervalos-this.pageActual-((this.numMaxIntervalos/2)-1)<0)){
				this.primeraPagina = this.numIntervalos-this.numMaxIntervalos+1;
			}
		}		
		// Si el número de intervalos es igual o menor al mínimo
		else{
			this.numMaxIntervalos = numIntervalos;
			this.primeraPagina=1;
		}
	}
	
	private void preparePagerMethod(HttpServletRequest request, long numElements, int page, String attrName, String attr) {
		
		//Creamos la acción correspondiente
		this.action = DoUrl(request, attrName, attr);
		
		// Creamos el bean a la que accederá la jsp y que 
		// contendra los datos necesarios.
		this.pagerBean = new PagerBean();
		this.numElements = numElements;
		this.pageActual = page;
		
		// Calculamos el numero de intervalos que necesitamos.
		this.numIntervalos = CalcularIntervalosTotal();
		this.paginaMaxima = this.numIntervalos;
		
		// Si el numero de intervalos es mayor que el máximo permitido
		if(numIntervalos>numMaxIntervalos){
			this.primeraPagina = this.pageActual-(this.numMaxIntervalos/2);
			
			// La primera página obligatoriamente es 1 o más
			if(this.primeraPagina<=1){
				this.primeraPagina=1;
			}
			
			// Si el número máximo de intervalos es impar
			else if(((this.numMaxIntervalos)%2==1)&&(this.numIntervalos-this.pageActual-(this.numMaxIntervalos/2)<0)){			
				// Al ser impar no podemos eleminar el 2 que multiplica con el 2 que divide
				this.primeraPagina = this.numIntervalos-2*(this.numMaxIntervalos/2);	
			}		
			// Si el número máximo de intervalos es par
			else if(((this.numMaxIntervalos)%2==0)&&(this.numIntervalos-this.pageActual-((this.numMaxIntervalos/2)-1)<0)){
				this.primeraPagina = this.numIntervalos-this.numMaxIntervalos+1;
			}
		}		
		// Si el número de intervalos es igual o menor al mínimo
		else{
			this.numMaxIntervalos = numIntervalos;
			this.primeraPagina=1;
		}
	}
	
	/**
	 * Método para crear la url
	 * @param request HttpServletRequest de la acción.
	 * @return String la url ya formateada
	 */
	private String DoUrl(HttpServletRequest request, String attrName, String attr){
		
		// Cogemos el path
		String path = request.getRequestURI();
		
		// Recuperamos los parametros
		String parametres = request.getQueryString();
		
		// Ponemos despues del path el interrogante para los parametros
		String completPath = path + "?";
		
		// Si ya tenia parametros
		if(parametres!=null){
			// Miramos si tenia el parametro page
			int index = parametres.indexOf("page");
			
			// Si tenia el parametro page lo hemos de borrar
			if(index!=-1){
				// Borramos el parametro page, siempre es el último
				parametres = parametres.substring(0, index);
			}		
			// Si la URL no acaba con & lo ponemos
			if(!parametres.endsWith("&")){
				completPath+= parametres+"&";
			}else{
				completPath+= parametres;
			}
		}else{
			completPath+=attrName+"="+attr+"&";
		}
		return completPath;
	}
	
	/**
	 * Método para crear la url
	 * @param request HttpServletRequest de la acción.
	 * @return String la url ya formateada
	 */
	private String DoUrl(HttpServletRequest request){
		
		// Cogemos el path
		String path = request.getRequestURI();
		
		// Recuperamos los parametros
		String parametres = request.getQueryString();
		
		// Ponemos despues del path el interrogante para los parametros
		String completPath = path + "?";
		
		// Si ya tenia parametros
		if(parametres!=null){
			// Miramos si tenia el parametro page
			int index = parametres.indexOf("page");
			
			// Si tenia el parametro page lo hemos de borrar
			if(index!=-1){
				// Borramos el parametro page, siempre es el último
				parametres = parametres.substring(0, index);
			}		
			// Si la URL no acaba con & lo ponemos
			if(!parametres.endsWith("&")){
				completPath+= parametres+"&";
			}else{
				completPath+= parametres;
			}
		}
		return completPath;
	}
	
	/**
	 * Método que calcula los intervalos que saldrán en la jsp.
	 * @return numIntervalos
	 */
	private int CalcularIntervalosTotal(){
		// Redondeado hacia arriba (2.1 -> 3).
		return (int)Math.ceil((double)this.numElements/this.numMaxElementsPage);
	}
	
	/**
	 * Retorna un String[] con lo que saldra en la jsp.
	 * @return intervalo
	 */
	private String[] getIntervalo(){
		String[] intervalo = new String[numMaxIntervalos+4];
		
		// Ponemos el Primero y anterior
		intervalo[0] = "pag.list.first";
		intervalo[1] = "pag.lint.before";
		
		// Rellenamos los siguientes con los números de paginas que correspondan
		int pagina = this.primeraPagina;
		for(int i=1;i<numMaxIntervalos+1;i++){
			intervalo[i+1] = pagina+"";
			pagina++;
		}		
		// Ponemos el Siguiente y último
		intervalo[numMaxIntervalos+2] = "pag.list.next";
		intervalo[numMaxIntervalos+3] = "pag.list.last";
		
		return intervalo;
	}
	
	/**
	 * Retorna un int[] con los enlaces a las páginas.
	 * Así en la posición 0 tenemos en la jsp intervalo[]
	 * y el enlace a la pagina que hay en paginaIntervalo[]
	 * @return paginaIntervalo
	 */
	private int[] getPaginaIntervalo(){
		int[] paginaIntervalo = new int[this.numMaxIntervalos+4];
		
		// Rellenamos la página de enlace de las primera páginas
		// El enlace Primero siempre apuntará a la primera página
		paginaIntervalo[0] = 1;
		
		// La página anterior apuntará a la pagina que estamos -1
		// Recordar que si estamos en la página 1, nos pondrá un enlace
		// a la página 0, la jsp ya se encargará de no poner el enlace.
		paginaIntervalo[1] = this.pageActual-1;
		
		// Ponemos los enlaces que correponde a cada uno
		this.numElementsIntervalo = 0;
		int pagina = this.primeraPagina;
		for(int i=1;i<numMaxIntervalos+1;i++){
			paginaIntervalo[i+1] = pagina;
			pagina++;
			this.numElementsIntervalo++;
		}	
		// Ponemos los enlaces en Siguiente, que será la actual mas uno.
		// Nota: Si estamos en la úiltima página, es la jsp la encargada de
		// no hacer el enlace
		paginaIntervalo[numMaxIntervalos+2] = this.pageActual+1;
		
		// Ponemos el enlace al la última página
		paginaIntervalo[numMaxIntervalos+3] = this.paginaMaxima;
		return paginaIntervalo;
	}
	
	/**
	 * Método privado que crea el Bean con todos los datos necesarios
	 * para que la jsp pueda pintar la paginación.
	 * @see com.lavinia.utils.pager.PagerBean
	 * @return pagerBean
	 */
	public PagerBean doPagerBean(){
		
		// Rellenamos el pagerBean con todos los datos que necesita
		// y que hemos calculado anteriormente.
		this.pagerBean.setNumElements(this.numElements);
		this.pagerBean.setIntervalo(getIntervalo());
		this.pagerBean.setPaginaIntervalo(getPaginaIntervalo());
		this.pagerBean.setAction(this.action);
		this.pagerBean.setPageActual(this.pageActual);
		this.pagerBean.setNumElementsIntervalo(this.numElementsIntervalo);
		this.pagerBean.setPaginaMaxima(this.paginaMaxima);
		
		return this.pagerBean;
	}

	/**
	 * Método que nos retorna el numero maximo de elementos por pagina
	 * @return numMaxElementsPage
	 */
	public int getNumMaxElementsPage() {
		return numMaxElementsPage;
	}
	
	public void setNumMaxElementsPage(int numMaxElementsPage) {
		this.numMaxElementsPage = numMaxElementsPage;
	}

	public String getAttr() {
		return attr;
	}

	public void setAttr(String attr) {
		this.attr = attr;
	}

	public String getAttrName() {
		return attrName;
	}

	public void setAttrName(String attrName) {
		this.attrName = attrName;
	}
}
