package com.selya.store.persistance;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Carreto entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Carreto
 * @author MyEclipse Persistence Tools
 */

public class CarretoDAO extends BaseHibernateDAO {
	private static final Logger log = Logger.getLogger(CarretoDAO.class);

	// property constants

	public void save(Carreto transientInstance) {
		log.debug("saving Carreto instance");
		try {
			getSession().close();
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Carreto persistentInstance) {
		log.debug("deleting Carreto instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Carreto findById(Long id) {
		log.debug("getting Carreto instance with id: " + id);
		try {
			getSession().clear();
			Carreto instance = (Carreto) getSession().get(
					"com.selya.store.persistance.Carreto", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Carreto> findByExample(Carreto instance) {
		log.debug("finding Carreto instance by example");
		try {
			getSession().clear();
			List<Carreto> results = (List<Carreto>) getSession()
					.createCriteria("com.selya.store.persistance.Carreto")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Carreto instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Carreto as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List findAll() {
		log.debug("finding all Carreto instances");
		try {
			getSession().clear();
			String queryString = "from Carreto";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Carreto merge(Carreto detachedInstance) {
		log.debug("merging Carreto instance");
		try {
			Transaction tx = getSession().beginTransaction();
			Carreto result = (Carreto) getSession().merge(detachedInstance);
			tx.commit();
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Carreto instance) {
		log.debug("attaching dirty Carreto instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	/**
	 * Method that return the import of a chart
	 * @param carretoId - identifier of chart to get the import
	 * @return import in € from the products in the chart
	 */
	public float importCarreto(long carretoId){
		
		float preu = 0;
		
		Carreto carreto = findById(carretoId);
		Set<Carretoproducte> productes = carreto.getCarretoproductes();
		Iterator<Carretoproducte> it = productes.iterator();
		
		while(it.hasNext()){
			Carretoproducte carretoProducte = it.next();
			int quantitat = carretoProducte.getQuantitat();
			preu += quantitat * carretoProducte.getProducte().preuPromocionat(carreto.getDataModificacio());
		}
		
		return preu;
	}
	
	/**
	 * Mètode que retorna l'últim carretoó introduït per l'usuari passat per paràmetre
	 * @param usuari
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Carreto getLastCarreto(Usuaris usuari){
		
		log.debug("finding last Carreto instance fon the user passed");
		try {
			getSession().clear();
			String queryString = "from Carreto as model where model.usuaris = ? and model.estatcarreto < 3 order by model.dataModificacio desc";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, usuari);
			List<Carreto> carretons = queryObject.list();
			if(carretons.size() > 0){
				return carretons.get(0);
			}
		} catch (RuntimeException re) {
			log.error("get last carreto failed", re);
			throw re;
		}
		
		return null;
	}
}