package com.selya.store.persistance;

import java.util.List;
import java.util.Set;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for Color
 * entities. Transaction control of the save(), update() and delete() operations
 * can directly support Spring container-managed transactions or they can be
 * augmented to handle user-managed Spring transactions. Each of these methods
 * provides additional information for how to configure it for the desired type
 * of transaction control.
 * 
 * @see com.selya.store.persistance.Color
 * @author MyEclipse Persistence Tools
 */

public class ColorDAO extends BaseHibernateDAO {
	private static final Logger log = Logger.getLogger(ColorDAO.class);
	// property constants
	public static final String DESCRIPCIO = "descripcio";

	public void save(Color transientInstance) {
		log.debug("saving Color instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			log.debug("save successful");
			tx.commit();
			getSession().close();
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Color persistentInstance) {
		log.debug("deleting Color instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Color findById(int colorId) {
		log.debug("getting Color instance with id: " + colorId);
		try {
			getSession().clear();
			Color instance = (Color) getSession().get(
					"com.selya.store.persistance.Color", colorId);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Color> findByExample(Color instance) {
		log.debug("finding Color instance by example");
		try {
			getSession().clear();
			List<Color> results = (List<Color>) getSession()
					.createCriteria("com.selya.store.persistance.Color")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Color instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Color as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Color> findByDescripcio(Object descripcio) {
		return findByProperty(DESCRIPCIO, descripcio);
	}

	public List findAll() {
		log.debug("finding all Color instances");
		try {
			getSession().clear();
			String queryString = "from Color";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Color merge(Color detachedInstance) {
		log.debug("merging Color instance");
		try {
			Color result = (Color) getSession().merge(detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Color instance) {
		log.debug("attaching dirty Color instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Color instance) {
		log.debug("attaching clean Color instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
}