package com.selya.store.persistance;

import java.util.Date;
import java.util.List;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Comentaris entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Comentaris
 * @author MyEclipse Persistence Tools
 */

public class ComentarisDAO extends BaseHibernateDAO {
	private static final Logger log = Logger
			.getLogger(ComentarisDAO.class);
	// property constants
	public static final String COMENTARI = "comentari";

	public void save(Comentaris transientInstance) {
		log.debug("saving Comentaris instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Comentaris persistentInstance) {
		log.debug("deleting Comentaris instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Comentaris findById(Long id) {
		log.debug("getting Comentaris instance with id: " + id);
		try {
			getSession().clear();
			Comentaris instance = (Comentaris) getSession().get(
					"com.selya.store.persistance.Comentaris", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Comentaris> findByExample(Comentaris instance) {
		log.debug("finding Comentaris instance by example");
		try {
			getSession().clear();
			List<Comentaris> results = (List<Comentaris>) getSession()
					.createCriteria("com.selya.store.persistance.Comentaris")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Comentaris instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Comentaris as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Comentaris> findByComentari(Object comentari) {
		return findByProperty(COMENTARI, comentari);
	}

	public List findAll() {
		log.debug("finding all Comentaris instances");
		try {
			getSession().clear();
			String queryString = "from Comentaris";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Comentaris merge(Comentaris detachedInstance) {
		log.debug("merging Comentaris instance");
		try {
			Comentaris result = (Comentaris) getSession().merge(
					detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Comentaris instance) {
		log.debug("attaching dirty Comentaris instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Comentaris instance) {
		log.debug("attaching clean Comentaris instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
	
	/**
	 * Métode que retorna els 10 últims comentaris
	 * @return
	 */
	public List<Comentaris> lastComments(){
		String sql = "from Comentaris order by data desc limit 10 ";
		Query queryObject = getSession().createQuery(sql);
		
		return queryObject.list();
	}
	
	/**
	 * Métode que retorna els comentaris ordenats de més nou a més antic
	 * @return
	 */
	public List<Comentaris> findAllDescSorted() {
		log.debug("finding all Comentaris instances");
		try {
			String queryString = "from Comentaris order by data";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}
}