package com.selya.store.persistance;

import java.util.List;
import java.util.Set;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Estatcarreto entities. Transaction control of the save(), update() and
 * delete() operations can directly support Spring container-managed
 * transactions or they can be augmented to handle user-managed Spring
 * transactions. Each of these methods provides additional information for how
 * to configure it for the desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Estatcarreto
 * @author MyEclipse Persistence Tools
 */

public class EstatcarretoDAO extends BaseHibernateDAO {
	private static final Logger log = Logger
			.getLogger(EstatcarretoDAO.class);
	// property constants
	public static final String DESCRIPCIO = "descripcio";

	public void save(Estatcarreto transientInstance) {
		log.debug("saving Estatcarreto instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Estatcarreto persistentInstance) {
		log.debug("deleting Estatcarreto instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Estatcarreto findById(java.lang.Integer id) {
		log.debug("getting Estatcarreto instance with id: " + id);
		try {
			getSession().clear();
			Estatcarreto instance = (Estatcarreto) getSession().get(
					"com.selya.store.persistance.Estatcarreto", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Estatcarreto> findByExample(Estatcarreto instance) {
		log.debug("finding Estatcarreto instance by example");
		try {
			getSession().clear();
			List<Estatcarreto> results = (List<Estatcarreto>) getSession()
					.createCriteria("com.selya.store.persistance.Estatcarreto")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Estatcarreto instance with property: "
				+ propertyName + ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Estatcarreto as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Estatcarreto> findByDescripcio(Object descripcio) {
		return findByProperty(DESCRIPCIO, descripcio);
	}

	public List findAll() {
		log.debug("finding all Estatcarreto instances");
		try {
			getSession().clear();
			String queryString = "from Estatcarreto";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Estatcarreto merge(Estatcarreto detachedInstance) {
		log.debug("merging Estatcarreto instance");
		try {
			Estatcarreto result = (Estatcarreto) getSession().merge(
					detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Estatcarreto instance) {
		log.debug("attaching dirty Estatcarreto instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Estatcarreto instance) {
		log.debug("attaching clean Estatcarreto instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
}