package com.selya.store.persistance;

import static javax.persistence.GenerationType.IDENTITY;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.selya.store.global.utils.Globals;

@Entity
@Table(name = "noticies", catalog = "tfc")
public class Noticies implements Serializable {

	protected Logger logger = Logger.getLogger(this.getClass().getName());
	
	private int id;
	private String titol;
	private String entradeta;
	private String noticia;
	private Date data;
	
	public Noticies(){
	}
	
	public Noticies(int id, String titol, String entradeta, String noticia, Date data){
		this.id = id;
		this.titol = titol;
		this.entradeta = entradeta;
		this.noticia = noticia;
		this.data = data;
	}

	@Id
	@GeneratedValue(strategy = IDENTITY)
	@Column(name = "Id", unique = true, nullable = false)
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	@Column(name = "Titol", nullable = false, length = 65535)
	public String getTitol() {
		return titol;
	}

	public void setTitol(String titol) {
		this.titol = titol;
	}
	
	@Column(name = "Entradeta", nullable = false, length = 65535)
	public String getEntradeta() {
		return entradeta;
	}

	public void setEntradeta(String entradeta) {
		this.entradeta = entradeta;
	}

	@Column(name = "Noticia", nullable = false, length = 65535)
	public String getNoticia() {
		return noticia;
	}

	public void setNoticia(String noticia) {
		this.noticia = noticia;
	}

	@Temporal(TemporalType.DATE)
	@Column(name = "Data", nullable = false, length = 10)
	public Date getData() {
		return data;
	}

	public void setData(Date data) {
		this.data = data;
	}
	
	/**
	 * Métode que retorna el títol en l'idioma passat per paràmetre
	 * @param language
	 * @return
	 */
	public String getTitol(String language){
		
		if (this.titol != null && !this.titol.equals("")) {
			String nombre=null;
			try {
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
			    InputSource is = new InputSource();
			    is.setCharacterStream(new StringReader(titol));
			    Document doc = db.parse(is);
				NodeList nodeLst = doc.getElementsByTagName("data");
				for (int s = 0; s < nodeLst.getLength(); s++) {

				    Node fstNode = nodeLst.item(s);
				    
				    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				  
				      Element fstElmnt = (Element) fstNode;
				      NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("language");
				      Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
				      NodeList fstNm = fstNmElmnt.getChildNodes();
				      if (fstNm.item(0).getNodeValue().equals(language)) {
				    	  NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("content"); 
				    	  Element lstNmElmnt = (Element) lstNmElmntLst.item(0);
					      NodeList lstNm = lstNmElmnt.getChildNodes();
					      nombre = ((Node) lstNm.item(0)).getNodeValue();
				      }
				    }

				  }
			} catch (SAXException e) {
				logger.error(e.getMessage(),e);
			} catch (IOException e) {
				logger.error(e.getMessage(),e);
			} catch (ParserConfigurationException e) {
				logger.error(e.getMessage(),e);
			} catch (Exception e) {
				logger.error(e.getMessage(),e);
			}
			return nombre;
			
		}
		else {
			return "";
		}
	}
	
	/**
	 * Mètode que assigna el títol per aml'idioma passat per paràmetre a la notícia
	 * @param language
	 * @param descripcio
	 */
	public void setTitol(String language, String descripcio){
		
		if (this.titol != null && !this.titol.equals("")) {
			try {
				boolean idiomaFinded = false;
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
			    InputSource is = new InputSource();
			    is.setCharacterStream(new StringReader(this.titol));
			    Document doc = db.parse(is);
				NodeList nodeLst = doc.getElementsByTagName("data");
				for (int s = 0; s < nodeLst.getLength(); s++) {

				    Node fstNode = nodeLst.item(s);
				    
				    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				  
				      Element fstElmnt = (Element) fstNode;
				      NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("language");
				      Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
				      NodeList fstNm = fstNmElmnt.getChildNodes();
				      if (fstNm.item(0).getNodeValue().equals(language)) {
				    	  idiomaFinded = true;
				    	  NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("content"); 
				    	  Element lstNmElmnt = (Element) lstNmElmntLst.item(0);
					      NodeList lstNm = lstNmElmnt.getChildNodes();
					      ((Node) lstNm.item(0)).setNodeValue(descripcio);
				      }
				    }

				  }
				if(!idiomaFinded){
					 StringBuffer sb = new StringBuffer(this.titol);
					 int ix = sb.indexOf("</data>");
					 sb.insert(ix+7, Globals.setDataTag(descripcio, language));
					 this.titol = sb.toString();
			      }else{
			    	  Source source = new DOMSource(doc);
			    	  StringWriter stringWriter = new StringWriter();
			    	  Result result = new StreamResult(stringWriter);
			    	  TransformerFactory factory = TransformerFactory.newInstance();
			    	  Transformer transformer = factory.newTransformer();
			    	  transformer.transform(source, result);
			    	  this.titol = stringWriter.getBuffer().toString();
			      }
			} catch (SAXException e) {
				logger.error(e.getMessage(),e);
			} catch (IOException e) {
				logger.error(e.getMessage(),e);
			} catch (ParserConfigurationException e) {
				logger.error(e.getMessage(),e);
			} catch (Exception e) {
				logger.error(e.getMessage(),e);
			}
		}
		else {
			StringBuffer sb = new StringBuffer(10);
			sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			sb.append("<root>");
			sb.append(Globals.setDataTag(descripcio, language));
			sb.append("</root>");
			this.titol = sb.toString();
		}
	}
	
	/**
	 * Métode que retorna l'entradeta en l'idioma passat per paràmetre
	 * @param language
	 * @return
	 */
	public String getEntradeta(String language){
		
		if (this.entradeta != null && !this.entradeta.equals("")) {
			String nom=null;
			try {
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
			    InputSource is = new InputSource();
			    is.setCharacterStream(new StringReader(entradeta));
			    Document doc = db.parse(is);
				NodeList nodeLst = doc.getElementsByTagName("data");
				for (int s = 0; s < nodeLst.getLength(); s++) {

				    Node fstNode = nodeLst.item(s);
				    
				    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				  
				      Element fstElmnt = (Element) fstNode;
				      NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("language");
				      Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
				      NodeList fstNm = fstNmElmnt.getChildNodes();
				      if (fstNm.item(0).getNodeValue().equals(language)) {
				    	  NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("content"); 
				    	  Element lstNmElmnt = (Element) lstNmElmntLst.item(0);
					      NodeList lstNm = lstNmElmnt.getChildNodes();
					      nom = ((Node) lstNm.item(0)).getNodeValue();
				      }
				    }

				  }
			} catch (SAXException e) {
				logger.error(e.getMessage(),e);
			} catch (IOException e) {
				logger.error(e.getMessage(),e);
			} catch (ParserConfigurationException e) {
				logger.error(e.getMessage(),e);
			} catch (Exception e) {
				logger.error(e.getMessage(),e);
			}
			return nom;
			
		}
		else {
			return "";
		}
	}
	
	/**
	 * Mètode que assigna l'entradeta per aml'idioma passat per paràmetre a la notícia
	 * @param language
	 * @param descripcio
	 */
	public void setEntradeta(String language, String descripcio){
		
		if (this.entradeta != null && !this.entradeta.equals("")) {
			try {
				boolean idiomaFinded = false;
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
			    InputSource is = new InputSource();
			    is.setCharacterStream(new StringReader(this.entradeta));
			    Document doc = db.parse(is);
				NodeList nodeLst = doc.getElementsByTagName("data");
				for (int s = 0; s < nodeLst.getLength(); s++) {

				    Node fstNode = nodeLst.item(s);
				    
				    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				  
				      Element fstElmnt = (Element) fstNode;
				      NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("language");
				      Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
				      NodeList fstNm = fstNmElmnt.getChildNodes();
				      if (fstNm.item(0).getNodeValue().equals(language)) {
				    	  idiomaFinded = true;
				    	  NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("content"); 
				    	  Element lstNmElmnt = (Element) lstNmElmntLst.item(0);
					      NodeList lstNm = lstNmElmnt.getChildNodes();
					      ((Node) lstNm.item(0)).setNodeValue(descripcio);
				      }
				    }

				  }
				if(!idiomaFinded){
					 StringBuffer sb = new StringBuffer(this.entradeta);
					 int ix = sb.indexOf("</data>");
					 sb.insert(ix+7, Globals.setDataTag(descripcio, language));
					 this.entradeta = sb.toString();
			      }else{
			    	  Source source = new DOMSource(doc);
			    	  StringWriter stringWriter = new StringWriter();
			    	  Result result = new StreamResult(stringWriter);
			    	  TransformerFactory factory = TransformerFactory.newInstance();
			    	  Transformer transformer = factory.newTransformer();
			    	  transformer.transform(source, result);
			    	  this.entradeta = stringWriter.getBuffer().toString();
			      }
			} catch (SAXException e) {
				logger.error(e.getMessage(),e);
			} catch (IOException e) {
				logger.error(e.getMessage(),e);
			} catch (ParserConfigurationException e) {
				logger.error(e.getMessage(),e);
			} catch (Exception e) {
				logger.error(e.getMessage(),e);
			}
		}
		else {
			StringBuffer sb = new StringBuffer(10);
			sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			sb.append("<root>");
			sb.append(Globals.setDataTag(descripcio, language));
			sb.append("</root>");
			this.entradeta = sb.toString();
		}
	}
	
	/**
	 * Métode que retorna el cos de la notícia en l'idioma passat per paràmetre
	 * @param language
	 * @return
	 */
	public String getNoticia(String language){
		
		if (this.noticia != null && !this.noticia.equals("")) {
			String nom=null;
			try {
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
			    InputSource is = new InputSource();
			    is.setCharacterStream(new StringReader(noticia));
			    Document doc = db.parse(is);
				NodeList nodeLst = doc.getElementsByTagName("data");
				for (int s = 0; s < nodeLst.getLength(); s++) {

				    Node fstNode = nodeLst.item(s);
				    
				    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				  
				      Element fstElmnt = (Element) fstNode;
				      NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("language");
				      Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
				      NodeList fstNm = fstNmElmnt.getChildNodes();
				      if (fstNm.item(0).getNodeValue().equals(language)) {
				    	  NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("content"); 
				    	  Element lstNmElmnt = (Element) lstNmElmntLst.item(0);
					      NodeList lstNm = lstNmElmnt.getChildNodes();
					      nom = ((Node) lstNm.item(0)).getNodeValue();
				      }
				    }

				  }
			} catch (SAXException e) {
				logger.error(e.getMessage(),e);
			} catch (IOException e) {
				logger.error(e.getMessage(),e);
			} catch (ParserConfigurationException e) {
				logger.error(e.getMessage(),e);
			} catch (Exception e) {
				logger.error(e.getMessage(),e);
			}
			return nom;
			
		}
		else {
			return "";
		}
	}
	
	/**
	 * Mètode que assigna el cos de la notícia per aml'idioma passat per paràmetre a la notícia
	 * @param language
	 * @param descripcio
	 */
	public void setNoticia(String language, String descripcio){
		
		if (this.noticia != null && !this.noticia.equals("")) {
			try {
				boolean idiomaFinded = false;
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
			    InputSource is = new InputSource();
			    is.setCharacterStream(new StringReader(this.noticia));
			    Document doc = db.parse(is);
				NodeList nodeLst = doc.getElementsByTagName("data");
				for (int s = 0; s < nodeLst.getLength(); s++) {

				    Node fstNode = nodeLst.item(s);
				    
				    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				  
				      Element fstElmnt = (Element) fstNode;
				      NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("language");
				      Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
				      NodeList fstNm = fstNmElmnt.getChildNodes();
				      if (fstNm.item(0).getNodeValue().equals(language)) {
				    	  idiomaFinded = true;
				    	  NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("content"); 
				    	  Element lstNmElmnt = (Element) lstNmElmntLst.item(0);
					      NodeList lstNm = lstNmElmnt.getChildNodes();
					      ((Node) lstNm.item(0)).setNodeValue(descripcio);
				      }
				    }

				  }
				if(!idiomaFinded){
					 StringBuffer sb = new StringBuffer(this.noticia);
					 int ix = sb.indexOf("</data>");
					 sb.insert(ix+7, Globals.setDataTag(descripcio, language));
					 this.noticia = sb.toString();
			      }else{
			    	  Source source = new DOMSource(doc);
			    	  StringWriter stringWriter = new StringWriter();
			    	  Result result = new StreamResult(stringWriter);
			    	  TransformerFactory factory = TransformerFactory.newInstance();
			    	  Transformer transformer = factory.newTransformer();
			    	  transformer.transform(source, result);
			    	  this.noticia = stringWriter.getBuffer().toString();
			      }
			} catch (SAXException e) {
				logger.error(e.getMessage(),e);
			} catch (IOException e) {
				logger.error(e.getMessage(),e);
			} catch (ParserConfigurationException e) {
				logger.error(e.getMessage(),e);
			} catch (Exception e) {
				logger.error(e.getMessage(),e);
			}
		}
		else {
			StringBuffer sb = new StringBuffer(10);
			sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			sb.append("<root>");
			sb.append(Globals.setDataTag(descripcio, language));
			sb.append("</root>");
			this.noticia = sb.toString();
		}
	}
}
