package com.selya.store.persistance;

import java.util.List;
import java.util.Set;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Paisos entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Paisos
 * @author MyEclipse Persistence Tools
 */

public class PaisosDAO extends BaseHibernateDAO {
	private static final Logger log = Logger.getLogger(PaisosDAO.class);
	// property constants
	public static final String NOM = "nom";

	public void save(Paisos transientInstance) {
		log.debug("saving Paisos instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Paisos persistentInstance) {
		log.debug("deleting Paisos instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Paisos findById(java.lang.Integer id) {
		log.debug("getting Paisos instance with id: " + id);
		try {
			Paisos instance = (Paisos) getSession().get(
					"com.selya.store.persistance.Paisos", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Paisos> findByExample(Paisos instance) {
		log.debug("finding Paisos instance by example");
		try {
			List<Paisos> results = (List<Paisos>) getSession()
					.createCriteria("com.selya.store.persistance.Paisos")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Paisos instance with property: " + propertyName
				+ ", value: " + value);
		try {
			String queryString = "from Paisos as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Paisos> findByNom(Object nom) {
		return findByProperty(NOM, nom);
	}

	public List findAll() {
		log.debug("finding all Paisos instances");
		try {
			String queryString = "from Paisos";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Paisos merge(Paisos detachedInstance) {
		log.debug("merging Paisos instance");
		try {
			Paisos result = (Paisos) getSession().merge(detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Paisos instance) {
		log.debug("attaching dirty Paisos instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Paisos instance) {
		log.debug("attaching clean Paisos instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
}