package com.selya.store.persistance;

import java.util.Date;
import java.util.List;
import java.util.Set;

import org.hibernate.Criteria;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Producte entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Producte
 * @author MyEclipse Persistence Tools
 */

public class ProducteDAO extends BaseHibernateDAO {
	private static final Logger log = Logger
			.getLogger(ProducteDAO.class);
	// property constants
	public static final String REF = "ref";
	public static final String TITOL = "titol";
	public static final String DESCRIPCIO = "descripcio";
	public static final String PREU = "preu";
	public static final String STOCK = "stock";
	public static final String IMATGE1 = "imatge1";
	public static final String IMATGE2 = "imatge2";
	public static final String IMATGE3 = "imatge3";

	public void save(Producte transientInstance) {
		log.debug("saving Producte instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Producte persistentInstance) {
		log.debug("deleting Producte instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Producte findById(Long id) {
		log.debug("getting Producte instance with id: " + id);
		try {
			getSession().clear();
			Producte instance = (Producte) getSession().get(
					"com.selya.store.persistance.Producte", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Producte> findByExample(Producte instance) {
		log.debug("finding Producte instance by example");
		try {
			getSession().clear();
			List<Producte> results = (List<Producte>) getSession()
					.createCriteria("com.selya.store.persistance.Producte")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Producte instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Producte as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}
	
	public List findLikeByTitelAndRef(String text) {
		log.debug("finding Producte instance with title: " + text
				+ ", ref: " + text);
		try {
			getSession().clear();
			Query query = getSession().createQuery("from Producte p where p.titol like :text or p.ref like :text");
			return query.setParameter("text", text + "%").list();
		} catch (RuntimeException re) {
			log.error("find by property title and ref", re);
			throw re;
		}
	}

	public List<Producte> findByRef(Object ref) {
		return findByProperty(REF, ref);
	}

	public List<Producte> findByTitol(Object titol) {
		return findByProperty(TITOL, titol);
	}

	public List<Producte> findByDescripcio(Object descripcio) {
		return findByProperty(DESCRIPCIO, descripcio);
	}

	public List<Producte> findByPreu(Object preu) {
		return findByProperty(PREU, preu);
	}

	public List<Producte> findByStock(Object stock) {
		return findByProperty(STOCK, stock);
	}

	public List<Producte> findByImatge1(Object imatge1) {
		return findByProperty(IMATGE1, imatge1);
	}

	public List<Producte> findByImatge2(Object imatge2) {
		return findByProperty(IMATGE2, imatge2);
	}

	public List<Producte> findByImatge3(Object imatge3) {
		return findByProperty(IMATGE3, imatge3);
	}

	public List findAll() {
		log.debug("finding all Producte instances");
		try {
			getSession().clear();
			String queryString = "from Producte as model where model.producte is null ";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Producte merge(Producte detachedInstance) {
		log.debug("merging Producte instance");
		try {
			Producte result = (Producte) getSession().merge(detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Producte instance) {
		log.debug("attaching dirty Producte instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Producte instance) {
		log.debug("attaching clean Producte instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
	
	/**
	 * Métode que retorna els productes més venuts entre dues dates
	 * @param dataInici
	 * @param dataFi
	 * @return
	 */
	public List<Producte> topVendes(Date dataInici, Date dataFi){
		
		String hql = "select producte.* " +
						"from Producte as producte " +
						"where producte.Id in (" +
							"select pc.producteId " +
							"from ProducteCarreto as pc ";
		if(dataInici != null){
			hql += "where pc.carretoId in (" +
					"select Id " +
					"from Carreto as c " +
					"where c.dataCreacio >= " + dataInici;
			if(dataFi != null)
				hql += "and c.dataModificacio <= " + dataFi;
		}else if(dataFi != null)
			hql += "where pc.carretoId in (" +
					"select Id " +
					"from Carreto as c " +
					"where c.dataModificacio <= " + dataFi;
		hql += "group by pc.producteId order by count(pc.producteId) desc)";
		Query query = getSession().createQuery(hql);
		
		return query.list();
	}
	
	/**
	 * Métode que retorna els 10 últims productes inserits o actualitzats com a novetats
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Producte> novetats(){
		String sql = "from Producte order by data desc";
		Query query = getSession().createQuery(sql);
		
		List<Producte> productes = query.list();
		if(productes.size() > 10){
			return productes.subList(0, 9);
		}else{
			return productes;
		}
		
	}
	
	/**
	 * Métode que retorna els 10 últims productes inserits o actualitzats com a novetats
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Producte> carrousel(){
		String sql = "from Producte order by data desc";
		Query query = getSession().createQuery(sql);
		
		List<Producte> productes = query.list();
		if(productes.size() > 5){
			return productes.subList(0, 5);
		}else{
			return productes;
		}
		
	}
	
	/**
	 * Métode que retorna els productes de la categoría espacificada i que continguin 
	 * la cadena <code>searchText</code> en el títol o la descripció
	 * @param categoria
	 * @param searchText
	 * @return
	 */
	public List<Producte> searchProductes(Categories categoria, String searchText){
		String sql = "from Producte " +
				"where categories = ? " +
				"and (titol like ? " +
				"or descripcio like ?)";
		Query query = getSession().createQuery(sql);
		query.setParameter(0, categoria);
		query.setParameter(1, searchText);
		query.setParameter(2, searchText);
		return query.list();
	}
}