package com.selya.store.persistance;

import java.util.List;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Productecolor entities. Transaction control of the save(), update() and
 * delete() operations can directly support Spring container-managed
 * transactions or they can be augmented to handle user-managed Spring
 * transactions. Each of these methods provides additional information for how
 * to configure it for the desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Productecolor
 * @author MyEclipse Persistence Tools
 */

public class ProductecolorDAO extends BaseHibernateDAO {
	private static final Logger log = Logger
			.getLogger(ProductecolorDAO.class);

	// property constants

	public void save(Productecolor transientInstance) {
		log.debug("saving Productecolor instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Productecolor persistentInstance) {
		log.debug("deleting Productecolor instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Productecolor findById(com.selya.store.persistance.ProductecolorId id) {
		log.debug("getting Productecolor instance with id: " + id);
		try {
			getSession().clear();
			Productecolor instance = (Productecolor) getSession().get(
					"com.selya.store.persistance.Productecolor", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Productecolor> findByExample(Productecolor instance) {
		log.debug("finding Productecolor instance by example");
		try {
			getSession().clear();
			List<Productecolor> results = (List<Productecolor>) getSession()
					.createCriteria("com.selya.store.persistance.Productecolor")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Productecolor instance with property: "
				+ propertyName + ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Productecolor as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List findAll() {
		log.debug("finding all Productecolor instances");
		try {
			getSession().clear();
			String queryString = "from Productecolor";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Productecolor merge(Productecolor detachedInstance) {
		log.debug("merging Productecolor instance");
		try {
			Productecolor result = (Productecolor) getSession().merge(
					detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Productecolor instance) {
		log.debug("attaching dirty Productecolor instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Productecolor instance) {
		log.debug("attaching clean Productecolor instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
	
	/**
	 * Mètode que elimina tots els colors d'un producte
	 * @param productId
	 */
	public void deleteProductProps(long productId){
		log.debug("delete colors fron the product with Id");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().createQuery("delete from Productecolor where producteId = :productId")
		    .setParameter("productId", productId)
		    .executeUpdate();
			tx.commit();
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}
}