package com.selya.store.persistance;

import static org.hibernate.criterion.Example.create;

import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import com.selya.store.global.utils.StringUtils;


/**
 * A data access object (DAO) providing persistence and search support for
 * Usuaris entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Usuaris
 * @author MyEclipse Persistence Tools
 */

public class UsuarisDAO extends BaseHibernateDAO {
	private static final Logger log = Logger.getLogger(UsuarisDAO.class);
	// property constants
	public static final String NOM = "nom";
	public static final String COGNOM1 = "cognom1";
	public static final String COGNOM2 = "cognom2";
	public static final String CARRER = "carrer";
	public static final String CP = "cp";
	public static final String POBLACIO = "poblacio";
	public static final String PROVINCIA = "provincia";
	public static final String USUARI = "usuari";
	public static final String PSW = "psw";
	public static final String EMAIL = "email";
	public static final String TELEFON = "telefon";

	public void save(Usuaris transientInstance) {
		log.debug("saving Usuaris instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Usuaris persistentInstance) {
		log.debug("deleting Usuaris instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Usuaris findById(Long id) {
		log.debug("getting Usuaris instance with id: " + id);
		try {
			getSession().clear();
			Usuaris instance = (Usuaris) getSession().get(
					"com.selya.store.persistance.Usuaris", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Usuaris> findByExample(Usuaris instance) {
		log.debug("finding Usuaris instance by example");
		try {
			getSession().clear();
			List<Usuaris> results = (List<Usuaris>) getSession()
					.createCriteria("com.selya.store.persistance.Usuaris")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Usuaris instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Usuaris as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Usuaris> findByNom(Object nom) {
		return findByProperty(NOM, nom);
	}

	public List<Usuaris> findByCognom1(Object cognom1) {
		return findByProperty(COGNOM1, cognom1);
	}

	public List<Usuaris> findByCognom2(Object cognom2) {
		return findByProperty(COGNOM2, cognom2);
	}

	public List<Usuaris> findByCarrer(Object carrer) {
		return findByProperty(CARRER, carrer);
	}

	public List<Usuaris> findByCp(Object cp) {
		return findByProperty(CP, cp);
	}

	public List<Usuaris> findByPoblacio(Object poblacio) {
		return findByProperty(POBLACIO, poblacio);
	}

	public List<Usuaris> findByProvincia(Object provincia) {
		return findByProperty(PROVINCIA, provincia);
	}

	public List<Usuaris> findByUsuari(Object usuari) {
		return findByProperty(USUARI, usuari);
	}

	public List<Usuaris> findByPsw(Object psw) {
		return findByProperty(PSW, psw);
	}

	public List<Usuaris> findByEmail(Object email) {
		return findByProperty(EMAIL, email);
	}

	public List<Usuaris> findByTelefon(Object telefon) {
		return findByProperty(TELEFON, telefon);
	}

	public List findAll() {
		log.debug("finding all Usuaris instances");
		try {
			getSession().clear();
			String queryString = "from Usuaris";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Usuaris merge(Usuaris detachedInstance) {
		log.debug("merging Usuaris instance");
		try {
			Usuaris result = (Usuaris) getSession().merge(detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Usuaris instance) {
		log.debug("attaching dirty Usuaris instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Usuaris instance) {
		log.debug("attaching clean Usuaris instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
	
	/**
	 * Métode que cerca els usuaris amb els paràmetres de cerca triats
	 * @param nom - nom dels usuaris a cercar
	 * @param cognom1 - primer cognom dels usuaris a cercar
	 * @param cognom2 - segon cognom dels usuaria a cercar
	 * @param email - adreça electrònica dels usuaris a cercar
	 * @param username - nom d'usuari dels usuaris a cercar
	 * @return <code>List<Usuaris></code> amb els usuaris que coincideixen amb els paràmetres sol·licitats
	 */
	public List<Usuaris> findUsuarisByParams(String nom, String cognom1, String cognom2, String email, String username){
		try {
			String queryString = "from Usuaris";
			if(StringUtils.isNotNull(nom) || StringUtils.isNotNull(cognom1) || StringUtils.isNotNull(cognom2) ||
					StringUtils.isNotNull(email) || StringUtils.isNotNull(username)){
				queryString += " where ";
				boolean init = true;
				if(StringUtils.isNotNull(nom)){
					queryString += NOM + " like '%" + nom + "%'";
					init = false;
				}
				if(StringUtils.isNotNull(cognom1)){
					if(init){
						queryString += COGNOM1 + " like '%" + cognom1 + "%'";
						init = false;
					}else{
						queryString += "  AND " + COGNOM1 + " like '%" + cognom1 + "%'";
					}
				}
				if(StringUtils.isNotNull(cognom2)){
					if(init){
						queryString += COGNOM2 + " like '%" + cognom2 + "%'";
						init = false;
					}else{
						queryString += " AND " + COGNOM2 + " like '%" + cognom2 + "%'";
					}
				}
				if(StringUtils.isNotNull(email)){
					if(init){
						queryString += EMAIL + " like '%" + email + "%'";
						init = false;
					}else{
						queryString += " AND " + EMAIL + " like '%" + email + "%'";
					}
				}
				if(StringUtils.isNotNull(username)){
					if(init){
						queryString += USUARI + " like '%" + username + "%'";
						init = false;
					}else{
						queryString += " AND " + USUARI + " like '%" + username + "%'";
					}
				}
				
			}
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}
	
	public List<Usuaris> topCompradors(Date dataInici, Date dataFi){
		
		String hql = "select usuari.* " +
						"from Usuaris as usuari" +
						"where usuari.Id in (" +
							"select cu.usuariId " +
							"from CarretoUsuari as cu ";
		if(dataInici != null){
			hql += "where cu.carretoId in (" +
					"select Id " +
					"from Carreto as c " +
					"where c.dataCreacio >= " + dataInici;
			if(dataFi != null)
				hql += "and c.dataModificacio <= " + dataFi;
		}else if(dataFi != null)
			hql += "where cu.carretoId in (" +
					"select Id " +
					"from Carreto as c " +
					"where c.dataModificacio <= " + dataFi;
		hql += "group by cu.usuariId order by count(cu.usuariId) desc)";
		Query query = getSession().createQuery(hql);
		
		return query.list();
	}
	
	/**
	 * Mètode que verifica que el nom d'usuari i la calu de pas corresponen
	 * @param username - nom d'usuari
	 * @param passsword - clau de pas de l'usuari
	 * @return
	 */
	public Usuaris loginUser(String username, String passsword){
		List<Usuaris> usuaris = findByUsuari(username);
		if(usuaris.size() >0){
			Usuaris usuari = usuaris.get(0);
			if(usuari.getPsw().trim().equalsIgnoreCase(StringUtils.hash(passsword).trim()))
				return usuari;
		}
		return null;
			
	}
}