package com.selya.store.persistance;

import java.util.Date;
import java.util.List;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Valoracio entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Valoracio
 * @author MyEclipse Persistence Tools
 */

public class ValoracioDAO extends BaseHibernateDAO {
	private static final Logger log = Logger
			.getLogger(ValoracioDAO.class);
	// property constants
	public static final String PUNTUACIO = "puntuacio";

	public void save(Valoracio transientInstance) {
		log.debug("saving Valoracio instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Valoracio persistentInstance) {
		log.debug("deleting Valoracio instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Valoracio findById(Long id) {
		log.debug("getting Valoracio instance with id: " + id);
		try {
			getSession().clear();
			Valoracio instance = (Valoracio) getSession().get(
					"com.selya.store.persistance.Valoracio", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Valoracio> findByExample(Valoracio instance) {
		log.debug("finding Valoracio instance by example");
		try {
			getSession().clear();
			List<Valoracio> results = (List<Valoracio>) getSession()
					.createCriteria("com.selya.store.persistance.Valoracio")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Valoracio instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Valoracio as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Valoracio> findByPuntuacio(Object puntuacio) {
		return findByProperty(PUNTUACIO, puntuacio);
	}

	public List findAll() {
		log.debug("finding all Valoracio instances");
		try {
			getSession().clear();
			String queryString = "from Valoracio";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Valoracio merge(Valoracio detachedInstance) {
		log.debug("merging Valoracio instance");
		try {
			Valoracio result = (Valoracio) getSession().merge(detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Valoracio instance) {
		log.debug("attaching dirty Valoracio instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Valoracio instance) {
		log.debug("attaching clean Valoracio instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
}