package com.selya.store.persistance;

import java.util.List;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;

import static org.hibernate.criterion.Example.create;
import org.apache.log4j.Logger;


/**
 * A data access object (DAO) providing persistence and search support for
 * Wishlist entities. Transaction control of the save(), update() and delete()
 * operations can directly support Spring container-managed transactions or they
 * can be augmented to handle user-managed Spring transactions. Each of these
 * methods provides additional information for how to configure it for the
 * desired type of transaction control.
 * 
 * @see com.selya.store.persistance.Wishlist
 * @author MyEclipse Persistence Tools
 */

public class WishlistDAO extends BaseHibernateDAO {
	private static final Logger log = Logger
			.getLogger(WishlistDAO.class);

	// property constants

	public void save(Wishlist transientInstance) {
		log.debug("saving Wishlist instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().saveOrUpdate(transientInstance);
			tx.commit();
			getSession().close();
			log.debug("save successful");
		} catch (RuntimeException re) {
			log.error("save failed", re);
			throw re;
		}
	}

	public void delete(Wishlist persistentInstance) {
		log.debug("deleting Wishlist instance");
		try {
			Transaction tx = getSession().beginTransaction();
			getSession().delete(persistentInstance);
			tx.commit();
			getSession().close();
			log.debug("delete successful");
		} catch (RuntimeException re) {
			log.error("delete failed", re);
			throw re;
		}
	}

	public Wishlist findById(com.selya.store.persistance.WishlistId id) {
		log.debug("getting Wishlist instance with id: " + id);
		try {
			getSession().clear();
			Wishlist instance = (Wishlist) getSession().get(
					"com.selya.store.persistance.Wishlist", id);
			return instance;
		} catch (RuntimeException re) {
			log.error("get failed", re);
			throw re;
		}
	}

	public List<Wishlist> findByExample(Wishlist instance) {
		log.debug("finding Wishlist instance by example");
		try {
			getSession().clear();
			List<Wishlist> results = (List<Wishlist>) getSession()
					.createCriteria("com.selya.store.persistance.Wishlist")
					.add(create(instance)).list();
			log.debug("find by example successful, result size: "
					+ results.size());
			return results;
		} catch (RuntimeException re) {
			log.error("find by example failed", re);
			throw re;
		}
	}

	public List findByProperty(String propertyName, Object value) {
		log.debug("finding Wishlist instance with property: " + propertyName
				+ ", value: " + value);
		try {
			getSession().clear();
			String queryString = "from Wishlist as model where model."
					+ propertyName + "= ?";
			Query queryObject = getSession().createQuery(queryString);
			queryObject.setParameter(0, value);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find by property name failed", re);
			throw re;
		}
	}

	public List findAll() {
		log.debug("finding all Wishlist instances");
		try {
			getSession().clear();
			String queryString = "from Wishlist";
			Query queryObject = getSession().createQuery(queryString);
			return queryObject.list();
		} catch (RuntimeException re) {
			log.error("find all failed", re);
			throw re;
		}
	}

	public Wishlist merge(Wishlist detachedInstance) {
		log.debug("merging Wishlist instance");
		try {
			Wishlist result = (Wishlist) getSession().merge(detachedInstance);
			log.debug("merge successful");
			return result;
		} catch (RuntimeException re) {
			log.error("merge failed", re);
			throw re;
		}
	}

	public void attachDirty(Wishlist instance) {
		log.debug("attaching dirty Wishlist instance");
		try {
			getSession().saveOrUpdate(instance);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}

	public void attachClean(Wishlist instance) {
		log.debug("attaching clean Wishlist instance");
		try {
			getSession().lock(instance, LockMode.NONE);
			log.debug("attach successful");
		} catch (RuntimeException re) {
			log.error("attach failed", re);
			throw re;
		}
	}
}